<?php

/**
 * @file
 * Contains the copy rule.
 */

/**
 * Implements of hook_trigger_info().
 *
 * Action to process order.
 */
function commerce_nodecopy_rules_action_info() {
  $actions = array();
  $actions['commerce_nodecopy_process_lineitem'] = array(
    'type' => 'commerce_order',
    'group' => t('Commerce line item'),
    'label' => t('Copy referenced nodes'),
    'parameter' => array(
      'commerce_line_item' => array(
        'type' => 'commerce_line_item',
        'label' => t('Line item that might contain a node reference field'),
      ),
    ),
    'callbacks' => array(
      'execute' => 'commerce_nodecopy_line_item_process',
    ),
  );
  return $actions;
}

/**
 * Process the line item and copy nodes, if any.
 */
function commerce_nodecopy_line_item_process($line_item) {
  if (!empty($line_item->commerce_product)) {
    module_load_include('inc', 'clone', 'clone.pages');
    $wrapper = entity_metadata_wrapper('commerce_line_item', $line_item);
    $fields = field_info_instances('commerce_product', $wrapper->commerce_product->type->value());

    // Check for correct instance.
    foreach ($fields as $field) {
      if (!empty($field['commerce_nodecopy_settings']['nodecopy_enabled'])) {
        // We have a enabled field, let's check for items.
        $field_name = $field['field_name'];
        // Both single & multiple fields should work.
        if ($nids = $wrapper->commerce_product->{$field_name}->raw()) {
          if (is_numeric($nids)) {
            $cloned_nids = commerce_nodecopy_save((int) $nids);
          }
          if (is_array($nids)) {
            foreach ($nids as $nid) {
              $cloned_nids[] = commerce_nodecopy_save((int) $nid);
            }
          }
        }
        // Write back to line item, if field exists there.
        if (is_array($line_item->{$field_name})) {
          $wrapper->{$field_name}->set($cloned_nids);
        }
      }
    }
  }
}

/**
 * Clones a node by directly saving it.
 * Almost a direct clone of clone-function in node_clone.module.
 */
function commerce_nodecopy_save($nid) {
  if (is_numeric($nid)) {
    global $user;

    $node = node_load($nid, NULL, TRUE);
    if (isset($node->nid)) {
      $original_node = clone $node;

      $node->nid = NULL;
      $node->vid = NULL;
      $node->tnid = NULL;
      $node->name = $user->name;
      $node->uid = $user->uid;
      $node->created = NULL;
      $node->menu = _commerce_nodecopy_menu_link($original_node);
      $node->book['mlid'] = NULL;
      $node->path = NULL;
      $node->files = array();
      $node->title = t('My new !title', array('!title' => $node->title));
      // Add an extra property as a flag.
      $node->clone_from_original_nid = $original_node->nid;

      module_invoke_all('commerce_nodecopy_presave', $node, $original_node);

      node_save($node);
      return $node->nid;
    }
  }
}

function _commerce_nodecopy_menu_link($node) {
  if (function_exists('menu_node_prepare')) {
    // This will fetch the existing menu link if the node had one.
    menu_node_prepare($node);

    if (!empty($node->menu['mlid'])) {
      $old_link = $node->menu;
      $link['link_title'] = t('Clone of !title', array('!title' => $old_link['link_title']));
      $link['plid'] = $old_link['plid'];
      $link['menu_name'] = $old_link['menu_name'];
      $link['weight'] = $old_link['weight'];
      $link['module'] = $old_link['module'];
      return $link;
    }
  }
  return NULL;
}
